@@ -29,11 +29,11 @@
 #include <CL/opencl.h>
 #include <poclu.h>
 
-#define N 128
+#define N 4
 
 #ifdef __cplusplus
 #  define CALLAPI "C"
-#else 
+#else
 #  define CALLAPI
 #endif
 
@@ -45,9 +45,10 @@ exec_dot_product_kernel (cl_context context, cl_device_id device,
 int
 main (int argc, char **argv)
 {
-  cl_float4 *srcA, *srcB;
-  cl_float *dst;
-  int i, err, spir, spirv;
+  cl_float4 *srcA = NULL;
+  cl_float4 *srcB = NULL;
+  cl_float *dst = NULL;
+  int i, err, spir, spirv, poclbin;
 
   cl_context context = NULL;
   cl_device_id device = NULL;
@@ -60,9 +61,12 @@ main (int argc, char **argv)
 
   spir = (argc > 1 && argv[1][0] == 's');
   spirv = (argc > 1 && argv[1][0] == 'v');
+  poclbin = (argc > 1 && argv[1][0] == 'b');
+  const char *explicit_binary_path = (poclbin && (argc > 2)) ? argv[2] : NULL;
 
   const char *basename = "example1";
-  err = poclu_load_program (context, device, basename, spir, spirv, &program);
+  err = poclu_load_program (context, device, basename, spir, spirv, poclbin,
+                            explicit_binary_path, NULL, &program);
   if (err != CL_SUCCESS)
     goto FINISH;
 
@@ -117,6 +121,9 @@ FINISH:
   CHECK_CL_ERROR (clReleaseCommandQueue (queue));
   CHECK_CL_ERROR (clUnloadPlatformCompiler (platform));
   CHECK_CL_ERROR (clReleaseContext (context));
+  free (srcA);
+  free (srcB);
+  free (dst);
 
   return err;
 }
