@@ -47,6 +47,11 @@
 
 #define POCL_FILENAME_LENGTH 1024
 
+/* official Khronos ID */
+#ifndef CL_KHRONOS_VENDOR_ID_POCL
+#define CL_KHRONOS_VENDOR_ID_POCL 0x10006
+#endif
+
 typedef struct _mem_mapping mem_mapping_t;
 /* represents a single buffer to host memory mapping */
 struct _mem_mapping {
@@ -369,21 +374,7 @@ struct _cl_command_node
   cl_int ready;
 };
 
-#ifndef LLVM_9_0
-#define LLVM_OLDER_THAN_9_0 1
-
-#ifndef LLVM_8_0
-#define LLVM_OLDER_THAN_8_0 1
-
-#ifndef LLVM_7_0
-#define LLVM_OLDER_THAN_7_0 1
-
-#ifndef LLVM_6_0
-#define LLVM_OLDER_THAN_6_0 1
-
-#endif
-#endif
-#endif
-#endif
+#define CLANG_MAJOR LLVM_MAJOR
+#include "_libclang_versions_checks.h"
 
 #endif /* POCL_H */
