@@ -21,10 +21,11 @@
    THE SOFTWARE.
 */
 
+#include <CL/cl.h>
+#include <string.h>
+
 #include "pocl_util.h"
 #include "pocl_image_util.h"
-#include "cl_platform.h"
-#include <string.h>
 
 extern CL_API_ENTRY cl_int CL_API_CALL
 POname(clEnqueueFillImage)(cl_command_queue  command_queue,
@@ -43,9 +44,6 @@ CL_API_SUFFIX__VERSION_1_2
 
   POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
 
-  POCL_RETURN_ERROR_ON((!command_queue->device->image_support), CL_INVALID_OPERATION,
-    "Device %s does not support images\n", command_queue->device->long_name);
-
   POCL_RETURN_ERROR_COND((image == NULL), CL_INVALID_MEM_OBJECT);
   POCL_RETURN_ERROR_COND((origin == NULL), CL_INVALID_VALUE);
   POCL_RETURN_ERROR_COND((region == NULL), CL_INVALID_VALUE);
@@ -54,8 +52,9 @@ CL_API_SUFFIX__VERSION_1_2
   POCL_RETURN_ERROR_ON((command_queue->context != image->context), CL_INVALID_CONTEXT,
       "image and command_queue are not from the same context\n");
 
-  POCL_RETURN_ERROR_ON((!image->is_image), CL_INVALID_MEM_OBJECT,
-                                                "image argument is not an image\n");
+  POCL_RETURN_ERROR_ON ((!image->is_image), CL_INVALID_MEM_OBJECT,
+                        "image argument is not an image\n");
+  POCL_RETURN_ON_UNSUPPORTED_IMAGE (image, command_queue->device);
 
   errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
                                         event_wait_list);
