@@ -82,5 +82,5 @@ POname(clEnqueueSVMFree) (cl_command_queue command_queue,
   return CL_SUCCESS;
 
 }
-POsym(clEnqueueSVMFree);
+POsym(clEnqueueSVMFree)
 
