@@ -41,6 +41,9 @@
 #include "llvm/Support/FileSystem.h"
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
+#ifndef LLVM_OLDER_THAN_11_0
+#include "llvm/Support/Alignment.h"
+#endif
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Target/TargetOptions.h"
 #include "llvm/Transforms/IPO.h"
@@ -107,12 +110,18 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
     }
   }
 
+#ifdef LLVM_OLDER_THAN_11_0
   llvm::StringRef Triple =
       (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
+#else
+  std::string Triple =
+      (sizeof(void *) == 8) ? "nvptx64-nvidia-cuda" : "nvptx-nvidia-cuda";
+#endif
 
   // Get NVPTX target.
   const llvm::Target *Target =
       llvm::TargetRegistry::lookupTarget(Triple, Error);
+
   if (!Target) {
     POCL_MSG_ERR("[CUDA] ptx-gen: failed to get target\n");
     POCL_MSG_ERR("%s\n", Error.c_str());
@@ -147,8 +156,13 @@ int pocl_ptx_gen(const char *BitcodeFilename, const char *PTXFilename,
   // Run passes.
   Passes.run(**Module);
 
+#ifdef LLVM_OLDER_THAN_11_0
   std::string PTX = PTXStream.str();
   return pocl_write_file(PTXFilename, PTX.c_str(), PTX.size(), 0, 0);
+#else
+  llvm::StringRef PTX = PTXStream.str();
+  return pocl_write_file(PTXFilename, PTX.data(), PTX.size(), 0, 0);
+#endif
 }
 
 // Add the metadata needed to mark a function as a kernel in PTX.
@@ -218,12 +232,25 @@ void fixPrintF(llvm::Module *Module) {
                                         OldPrintF->getBasicBlockList());
 
   // Create i32 to hold current argument index.
+#ifdef LLVM_OLDER_THAN_11_0
   llvm::AllocaInst *ArgIndexPtr =
       new llvm::AllocaInst(I32, 0, llvm::ConstantInt::get(I32, 1));
   ArgIndexPtr->insertBefore(&*NewPrintF->begin()->begin());
+#else
+  llvm::AllocaInst *ArgIndexPtr =
+      new llvm::AllocaInst(I32, 0, llvm::ConstantInt::get(I32, 1), llvm::Align(4));
+  ArgIndexPtr->insertBefore(&*NewPrintF->begin()->begin());
+#endif
+
+#ifdef LLVM_OLDER_THAN_11_0
   llvm::StoreInst *ArgIndexInit =
       new llvm::StoreInst(llvm::ConstantInt::get(I32, 0), ArgIndexPtr);
   ArgIndexInit->insertAfter(ArgIndexPtr);
+#else
+  llvm::StoreInst *ArgIndexInit =
+      new llvm::StoreInst(llvm::ConstantInt::get(I32, 0), ArgIndexPtr, false, llvm::Align(4));
+  ArgIndexInit->insertAfter(ArgIndexPtr);
+#endif
 
   // Replace calls to _cl_va_arg with reads from new i64 array argument.
   llvm::Function *VaArgFunc = Module->getFunction("__cl_va_arg");
@@ -239,9 +266,13 @@ void fixPrintF(llvm::Module *Module) {
         continue;
 
       // Get current argument index.
+#ifdef LLVM_OLDER_THAN_11_0
       llvm::LoadInst *ArgIndex = new llvm::LoadInst(ArgIndexPtr);
       ArgIndex->insertBefore(Call);
-
+#else
+      llvm::LoadInst *ArgIndex = new llvm::LoadInst(I32, ArgIndexPtr, "poclCudaLoad", false, llvm::Align(4));
+      ArgIndex->insertBefore(Call);
+#endif
       // Get pointer to argument data.
       llvm::Value *ArgOut = Call->getArgOperand(1);
       llvm::GetElementPtrInst *ArgIn =
@@ -254,18 +285,29 @@ void fixPrintF(llvm::Module *Module) {
       ArgOut = ArgOutBC;
 
       // Load argument.
+#ifdef LLVM_OLDER_THAN_11_0
       llvm::LoadInst *ArgValue = new llvm::LoadInst(ArgIn);
       ArgValue->insertAfter(ArgIn);
       llvm::StoreInst *ArgStore = new llvm::StoreInst(ArgValue, ArgOut);
       ArgStore->insertAfter(ArgOutBC);
-
+#else
+      llvm::LoadInst *ArgValue = new llvm::LoadInst(I64, ArgIn, "poclCudaArgLoad", false, llvm::Align(8));
+      ArgValue->insertAfter(ArgIn);
+      llvm::StoreInst *ArgStore = new llvm::StoreInst(ArgValue, ArgOut, false, llvm::Align(8));
+      ArgStore->insertAfter(ArgOutBC);
+#endif
       // Increment argument index.
       llvm::BinaryOperator *Inc = llvm::BinaryOperator::Create(
           llvm::BinaryOperator::Add, ArgIndex, llvm::ConstantInt::get(I32, 1));
       Inc->insertAfter(ArgIndex);
+
+#ifdef LLVM_OLDER_THAN_11_0
       llvm::StoreInst *StoreInc = new llvm::StoreInst(Inc, ArgIndexPtr);
       StoreInc->insertAfter(Inc);
-
+#else
+      llvm::StoreInst *StoreInc = new llvm::StoreInst(Inc, ArgIndexPtr, false, llvm::Align(4));
+      StoreInc->insertAfter(Inc);
+#endif
       // Remove call to _cl_va_arg.
       Call->eraseFromParent();
     }
@@ -288,9 +330,15 @@ void fixPrintF(llvm::Module *Module) {
 
     // Allocate array for arguments.
     // TODO: Deal with vector arguments.
+#ifdef LLVM_OLDER_THAN_11_0
     llvm::AllocaInst *Args =
         new llvm::AllocaInst(I64, 0, llvm::ConstantInt::get(I32, NumArgs));
     Args->insertBefore(Call);
+#else
+    llvm::AllocaInst *Args =
+        new llvm::AllocaInst(I64, 0, llvm::ConstantInt::get(I32, NumArgs), llvm::Align(8));
+    Args->insertBefore(Call);
+#endif
 
     // Loop over arguments (skipping format).
     for (unsigned A = 0; A < NumArgs; A++) {
@@ -322,8 +370,13 @@ void fixPrintF(llvm::Module *Module) {
       }
 
       // Store argument to i64 array.
+#ifdef LLVM_OLDER_THAN_11_0
       llvm::StoreInst *Store = new llvm::StoreInst(Arg, ArgPtr);
       Store->insertBefore(Call);
+#else
+      llvm::StoreInst *Store = new llvm::StoreInst(Arg, ArgPtr, false, llvm::Align(8));
+      Store->insertBefore(Call);
+#endif
     }
 
     // Fix address space of undef format values.
@@ -609,7 +662,6 @@ void convertPtrArgsToOffsets(llvm::Module *Module, const char *KernelName,
   llvm::ValueToValueMapTy VV;
   std::vector<std::pair<llvm::Instruction *, llvm::Instruction *>> ToInsert;
 
-  // Loop over arguments.
   bool NeedsArgOffsets = false;
   for (auto &Arg : Function->args()) {
     // Check for local memory pointer.
@@ -788,11 +840,21 @@ void mapLibDeviceCalls(llvm::Module *Module) {
     {"llvm.powi.f32", "__nv_powif"},
     {"llvm.powi.f64", "__nv_powi"},
 
-    // TODO: frexp
-    // TODO: ldexp
+    {"frexp", "__nv_frexp"},
+    {"frexpf", "__nv_frexpf"},
+
+    {"tgamma", "__nv_tgamma"},
+    {"tgammaf", "__nv_tgammaf"},
+
+    {"ldexp", "__nv_ldexp"},
+    {"ldexpf", "__nv_ldexpf"},
+
+    {"modf", "__nv_modf"},
+    {"modff", "__nv_modff"},
+
+    {"remquo", "__nv_remquo"},
+    {"remquof", "__nv_remquof"},
     // TODO: lgamma_r
-    // TODO: modf
-    // TODO: remquo
     // TODO: rootn
   };
   // clang-format on
