@@ -1,6 +1,6 @@
 /* pocl-hsa.c - driver for HSA supported devices.
 
-   Copyright (c) 2015-2019 Pekka Jääskeläinen
+   Copyright (c) 2015-2020 Pekka Jääskeläinen
                  2015 Charles Chen <ccchen@pllab.cs.nthu.edu.tw>
                       Shao-chung Wang <scwang@pllab.cs.nthu.edu.tw>
                  2015-2018 Michal Babej <michal.babej@tut.fi>
@@ -87,6 +87,7 @@
 #include "pocl_mem_management.h"
 #include "pocl_context.h"
 #include "pocl_spir.h"
+#include "pocl_local_size.h"
 
 #include <assert.h>
 #include <limits.h>
@@ -105,20 +106,6 @@
 
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
-/* TODO:
-   - fix pocl_llvm_generate_workgroup_function() to
-     generate the entire linked program.bc for HSA, not just a single kernel.
-   - AMD SDK samples. Requires work mainly in these areas:
-      - image support
-      - CL C++
-   - OpenCL printf() support
-   - get_global_offset() and global_work_offset param of clEnqueNDRker -
-     HSA kernel dispatch packet has no offset fields -
-     we must take care of it somewhere
-   - clinfo of Ubuntu crashes
-   - etc. etc.
-*/
-
 /* TODO: The kernel cache is never shrunk. We need a hook that is called back
    when clReleaseKernel is called to get a safe point where to release the
    kernel entry from the inmemory cache. */
@@ -127,7 +114,10 @@
 #define EVENT_LIST_SIZE 511
 
 typedef struct pocl_hsa_event_data_s {
-  void* actual_kernargs;
+  /* Address of the space where this kernel launch's arguments were stored. */
+  void *kernargs;
+  /* The location of the pocl context struct in the Agent's global mem. */
+  void *context;
   pthread_cond_t event_cond;
 } pocl_hsa_event_data_t;
 
@@ -236,7 +226,8 @@ void pocl_hsa_compile_kernel_native (_cl_command_node *cmd, cl_kernel kernel,
                                      cl_device_id device, int specialize);
 
 static void*
-pocl_hsa_malloc_account(pocl_global_mem_t *mem, size_t size, hsa_region_t r);
+pocl_hsa_malloc_account(pocl_global_mem_t *mem, size_t size, hsa_region_t r,
+                        int full_profile_agent);
 
 void
 pocl_hsa_init_device_ops(struct pocl_device_ops *ops)
@@ -258,6 +249,7 @@ pocl_hsa_init_device_ops(struct pocl_device_ops *ops)
   ops->unmap_mem = pocl_basic_unmap_mem;
   ops->memfill = pocl_basic_memfill;
   ops->copy = pocl_hsa_copy;
+  ops->compute_local_size = pocl_default_local_size_optimizer;
 
   ops->svm_free = pocl_hsa_svm_free;
   ops->svm_alloc = pocl_hsa_svm_alloc;
@@ -375,7 +367,7 @@ static struct _cl_device_id supported_hsa_devices[HSA_NUM_KNOWN_HSA_AGENTS]
                 .llvm_target_triplet
                 = (HSAIL_ENABLED ? "hsail64" : "amdgcn--amdhsa"),
                 .spmd = CL_TRUE,
-                .autolocals_to_args = CL_FALSE,
+                .autolocals_to_args = POCL_AUTOLOCALS_TO_ARGS_NEVER,
                 .device_alloca_locals = CL_FALSE,
                 .context_as_id = SPIR_ADDRESS_SPACE_GLOBAL,
                 .args_as_id = SPIR_ADDRESS_SPACE_GLOBAL,
@@ -387,7 +379,7 @@ static struct _cl_device_id supported_hsa_devices[HSA_NUM_KNOWN_HSA_AGENTS]
                 .endian_little = CL_TRUE,
                 .extensions = HSA_DEVICE_EXTENSIONS,
                 .device_side_printf = !HSAIL_ENABLED,
-                .printf_buffer_size = 16 * 1024 * 1024,
+                .printf_buffer_size = PRINTF_BUFFER_SIZE * 1024,
                 .preferred_wg_size_multiple = 64, // wavefront size on Kaveri
                 .preferred_vector_width_char = 4,
                 .preferred_vector_width_short = 2,
@@ -405,7 +397,9 @@ static struct _cl_device_id supported_hsa_devices[HSA_NUM_KNOWN_HSA_AGENTS]
                 .llvm_cpu = NULL,
                 .llvm_target_triplet = (HSAIL_ENABLED ? "hsail64" : NULL),
                 .spmd = CL_FALSE,
-                .autolocals_to_args = !HSAIL_ENABLED,
+                .autolocals_to_args
+                = (HSAIL_ENABLED ? POCL_AUTOLOCALS_TO_ARGS_NEVER
+                                 : POCL_AUTOLOCALS_TO_ARGS_ALWAYS),
                 .device_alloca_locals = CL_TRUE,
                 .context_as_id = SPIR_ADDRESS_SPACE_GLOBAL,
                 .args_as_id = SPIR_ADDRESS_SPACE_GLOBAL,
@@ -417,7 +411,7 @@ static struct _cl_device_id supported_hsa_devices[HSA_NUM_KNOWN_HSA_AGENTS]
                 .endian_little = !(WORDS_BIGENDIAN),
                 .extensions = HSA_DEVICE_EXTENSIONS,
                 .device_side_printf = !HSAIL_ENABLED,
-                .printf_buffer_size = 16 * 1024 * 1024,
+                .printf_buffer_size = PRINTF_BUFFER_SIZE * 1024,
                 .preferred_wg_size_multiple = 1,
                 /* We want to exploit the widest vector types in HSAIL
                    for the CPUs assuming they have some sort of SIMD ISE
@@ -597,7 +591,6 @@ init_dev_data (cl_device_id dev, int count)
   /* TODO check at runtime */
   d->have_wait_any = 1;
 #endif
-  HSA_CHECK (hsa_signal_create (1, 1, &d->agent, &d->nudge_driver_thread));
 
 #if AMD_HSA == 1
   if (dev->vendor_id == AMD_VENDOR_ID)
@@ -626,8 +619,6 @@ init_dev_data (cl_device_id dev, int count)
   if (dev->global_mem_size > 16 * 1024 * 1024 * (uint64_t)1024)
     dev->global_mem_size = dev->max_mem_alloc_size;
 
-  pocl_setup_device_for_system_memory (dev);
-
   HSA_CHECK (
       hsa_region_get_info (d->group_region, HSA_REGION_INFO_SIZE, &sizearg));
   dev->local_mem_size = sizearg;
@@ -638,6 +629,10 @@ init_dev_data (cl_device_id dev, int count)
 
   HSA_CHECK (hsa_agent_get_info (d->agent, HSA_AGENT_INFO_PROFILE,
                                  &d->agent_profile));
+
+  if (d->agent_profile == HSA_PROFILE_FULL)
+    pocl_setup_device_for_system_memory (dev);
+
   dev->profile = "FULL_PROFILE";
   dev->has_own_timer = CL_TRUE;
 
@@ -645,10 +640,13 @@ init_dev_data (cl_device_id dev, int count)
 
   if (dev->device_side_printf)
     {
-      d->printf_buffer = pocl_hsa_malloc_account (
-          dev->global_memory, dev->printf_buffer_size, d->global_region);
-      d->printf_write_pos = pocl_hsa_malloc_account (
-          dev->global_memory, sizeof (size_t), d->global_region);
+      d->printf_buffer = pocl_hsa_malloc_account
+        (dev->global_memory, dev->printf_buffer_size, d->global_region,
+         d->agent_profile == HSA_PROFILE_FULL);
+
+      d->printf_write_pos = pocl_hsa_malloc_account
+        (dev->global_memory, sizeof (size_t), d->global_region,
+         d->agent_profile == HSA_PROFILE_FULL);
     }
 
   d->exit_driver_thread = 0;
@@ -661,14 +659,14 @@ init_dev_data (cl_device_id dev, int count)
 cl_int
 pocl_hsa_init (unsigned j, cl_device_id dev, const char *parameters)
 {
-  pocl_init_cpu_device_infos (dev);
+  pocl_init_default_device_infos (dev);
 
   SETUP_DEVICE_CL_VERSION(HSA_DEVICE_CL_VERSION_MAJOR,
                           HSA_DEVICE_CL_VERSION_MINOR)
 
   dev->spmd = CL_TRUE;
   dev->arg_buffer_launcher = CL_FALSE;
-  dev->autolocals_to_args = CL_FALSE;
+  dev->autolocals_to_args = POCL_AUTOLOCALS_TO_ARGS_NEVER;
   dev->device_alloca_locals = CL_FALSE;
 
   dev->local_as_id = SPIR_ADDRESS_SPACE_LOCAL;
@@ -728,6 +726,12 @@ pocl_hsa_init (unsigned j, cl_device_id dev, const char *parameters)
 
   if (max_wg > 0)
     dev->max_work_group_size = max_wg;
+
+  /* Assume a small maximum work-group size indicates also the desire to
+     maximally utilize it. */
+  if (max_wg <= 256)
+    dev->ops->compute_local_size = pocl_wg_utilization_maximizer;
+
   if (AMD_HSA && dev->vendor_id == AMD_VENDOR_ID)
     {
 #if AMD_HSA == 1
@@ -810,26 +814,33 @@ pocl_hsa_init (unsigned j, cl_device_id dev, const char *parameters)
 }
 
 static void*
-pocl_hsa_malloc_account(pocl_global_mem_t *mem, size_t size, hsa_region_t r)
+pocl_hsa_malloc_account(pocl_global_mem_t *mem, size_t size, hsa_region_t r,
+                        int full_profile_agent)
 {
-  void *b = NULL;
-  if ((mem->total_alloc_limit - mem->currently_allocated) < size)
-    {
-      POCL_MSG_PRINT_INFO ("total alloc limit reached!");
-      return NULL;
-    }
+/* With full profile agents, we must account for other allocations from
+   the the same virtual memory space. With base profile, we can assume the
+   global memory chunk reported by the HSA runtime is an isolated area. */
 
-  if (hsa_memory_allocate(r, size, &b) != HSA_STATUS_SUCCESS)
+  void *b = NULL;
+  if (full_profile_agent)
+    {
+      if (mem->total_alloc_limit - mem->currently_allocated < size)
+        {
+          POCL_MSG_PRINT_INFO ("total alloc limit reached!");
+          return NULL;
+        }
+      /* FIXME: Not thread safe! */
+      mem->currently_allocated += size;
+      if (mem->max_ever_allocated < mem->currently_allocated)
+        mem->max_ever_allocated = mem->currently_allocated;
+      assert(mem->currently_allocated <= mem->total_alloc_limit);
+    }
+  else if (hsa_memory_allocate(r, size, &b) != HSA_STATUS_SUCCESS)
     {
       POCL_MSG_PRINT_INFO ("hsa_memory_allocate failed");
       return NULL;
     }
 
-  mem->currently_allocated += size;
-  if (mem->max_ever_allocated < mem->currently_allocated)
-    mem->max_ever_allocated = mem->currently_allocated;
-  assert(mem->currently_allocated <= mem->total_alloc_limit);
-
   if (b)
     POCL_MSG_PRINT_INFO("HSA malloc'ed : size %" PRIuS " @ %p\n", size, b);
 
@@ -855,19 +866,21 @@ pocl_hsa_malloc (cl_device_id device, cl_mem_flags flags, size_t size,
     {
       assert(host_ptr != NULL);
       if (d->agent_profile == HSA_PROFILE_FULL)
-	{
-	  POCL_MSG_PRINT_INFO
-	    ("HSA: CL_MEM_USE_HOST_PTR FULL profile: hsa_memory_register()\n");
-	  /* TODO bookkeeping of mem registrations. */
-	  hsa_memory_register(host_ptr, size);
-	  return host_ptr;
-	}
+        {
+          POCL_MSG_PRINT_INFO
+            ("HSA: CL_MEM_USE_HOST_PTR FULL profile: hsa_memory_register()\n");
+          /* TODO bookkeeping of mem registrations. */
+          hsa_memory_register(host_ptr, size);
+          return host_ptr;
+        }
       else
-	{
-	  POCL_MSG_PRINT_INFO
-	    ("HSA: CL_MEM_USE_HOST_PTR BASE profile: cached device copy\n");
-	  return pocl_hsa_malloc_account(mem, size, d->global_region);
-	}
+        {
+          POCL_MSG_PRINT_INFO
+            ("HSA: CL_MEM_USE_HOST_PTR BASE profile: cached device copy\n");
+          return pocl_hsa_malloc_account
+              (mem, size, d->global_region,
+               d->agent_profile == HSA_PROFILE_FULL);
+        }
     }
 
   if (flags & CL_MEM_COPY_HOST_PTR)
@@ -876,7 +889,10 @@ pocl_hsa_malloc (cl_device_id device, cl_mem_flags flags, size_t size,
                           " (CL_MEM_COPY_HOST_PTR)\n");
       assert(host_ptr != NULL);
 
-      b = pocl_hsa_malloc_account(mem, size, d->global_region);
+      void *b = NULL;
+      /* See above (*). */
+      b = pocl_hsa_malloc_account(mem, size, d->global_region,
+                                  d->agent_profile == HSA_PROFILE_FULL);
       if (b)
         hsa_memory_copy(b, host_ptr, size);
       return b;
@@ -884,7 +900,8 @@ pocl_hsa_malloc (cl_device_id device, cl_mem_flags flags, size_t size,
 
   assert(host_ptr == NULL);
   //POCL_MSG_PRINT_INFO("HSA: hsa_memory_allocate (ALLOC_HOST_PTR)\n");
-  return pocl_hsa_malloc_account(mem, size, d->global_region);
+  return pocl_hsa_malloc_account(mem, size, d->global_region,
+                                 d->agent_profile == HSA_PROFILE_FULL);
 }
 
 void
@@ -899,9 +916,13 @@ pocl_hsa_free (cl_device_id device, cl_mem memobj)
     hsa_memory_deregister(ptr, size);
   else
     {
-      pocl_global_mem_t *mem = device->global_memory;
-      assert(mem->currently_allocated >= size);
-      mem->currently_allocated -= size;
+      pocl_hsa_device_data_t *d = (pocl_hsa_device_data_t*)device->data;
+      if (d->agent_profile == HSA_PROFILE_FULL)
+        {
+          pocl_global_mem_t *mem = device->global_memory;
+          assert(mem->currently_allocated >= size);
+          mem->currently_allocated -= size;
+        }
       hsa_memory_free(ptr);
     }
   if (memobj->flags | CL_MEM_ALLOC_HOST_PTR)
@@ -974,12 +995,12 @@ pocl_hsa_alloc_mem_obj (cl_device_id device, cl_mem mem_obj, void *host_ptr)
 static void
 setup_kernel_args (pocl_hsa_device_data_t *d,
                    _cl_command_node *cmd,
-                   char *arg_space,
+                   pocl_hsa_event_data_t *event_data,
                    size_t max_args_size,
                    uint32_t *total_group_size)
 {
-  char *write_pos = arg_space;
-  const char *last_pos = arg_space + max_args_size;
+  char *write_pos = event_data->kernargs;
+  const char *last_pos = event_data->kernargs + max_args_size;
   cl_kernel kernel = cmd->command.run.kernel;
   pocl_kernel_metadata_t *meta = kernel->meta;
 
@@ -1062,7 +1083,7 @@ setup_kernel_args (pocl_hsa_device_data_t *d,
           POCL_MSG_PRINT_INFO (
               "arg %lu (global ptr) written to %lx val %lx arg offs %d\n", i,
               (uint64_t)write_pos, *(uint64_t *)write_pos,
-              (int)(write_pos - arg_space));
+              (int)(write_pos - (char*)event_data->kernargs));
           write_pos += sizeof (uint64_t);
         }
       else if (meta->arg_info[i].type == POCL_ARG_TYPE_IMAGE)
@@ -1083,32 +1104,33 @@ setup_kernel_args (pocl_hsa_device_data_t *d,
           POCL_MSG_PRINT_INFO (
               "arg %lu (scalar) written to %lx val %x offs %d\n", i,
               (uint64_t)write_pos, *(uint32_t *)al->value,
-              (int)(write_pos - arg_space));
+              (int)(write_pos - (char*)event_data->kernargs));
           write_pos += al->size;
         }
     }
 
   CHECK_AND_ALIGN_SPACE(sizeof (uint64_t));
 
-  /* Need to copy the context object to HSA allocated global memory
-     to ensure Base profile agents can access it. */
+  /* Copy the context object to HSA allocated global memory to ensure Base
+     profile agents can access it. */
 
-  void *ctx_ptr = pocl_hsa_malloc_account
+  event_data->context = pocl_hsa_malloc_account
     (d->device->global_memory, POCL_CONTEXT_SIZE (d->device->address_bits),
-     d->global_region);
+     d->global_region, d->agent_profile == HSA_PROFILE_FULL);
 
   if (d->device->address_bits == 64)
-    memcpy (ctx_ptr, &cmd->command.run.pc, sizeof (struct pocl_context));
+    memcpy (event_data->context, &cmd->command.run.pc, sizeof (struct pocl_context));
   else
-    POCL_CONTEXT_COPY64TO32 (ctx_ptr, &cmd->command.run.pc);
+    POCL_CONTEXT_COPY64TO32 (event_data->context, &cmd->command.run.pc);
 
-  memcpy (write_pos, &ctx_ptr, sizeof (ctx_ptr));
+  memcpy (write_pos, &event_data->context, sizeof (event_data->context));
   POCL_MSG_PRINT_INFO ("A %d-bit context object was written at %p offs %d\n",
-                       d->device->address_bits, ctx_ptr,
-                       (int)(write_pos - arg_space));
+                       d->device->address_bits, event_data->context,
+                       (int)(write_pos - (char*)event_data->kernargs));
   write_pos += sizeof (uint64_t);
 
-  /* MUST TODO: free the local buffers and ctx obj after finishing the kernel!
+  /* MUST TODO: free the local buffers after finishing the kernel in case of
+     host side allocation.
    */
 }
 
@@ -1481,7 +1503,10 @@ pocl_hsa_uninit (unsigned j, cl_device_id device)
     }
 
   if (device->device_side_printf)
-    hsa_memory_free (d->printf_buffer);
+    {
+      hsa_memory_free (d->printf_buffer);
+      hsa_memory_free (d->printf_write_pos);
+    }
 
   unsigned i;
   for (i = 0; i < HSA_KERNEL_CACHE_SIZE; i++)
@@ -1552,7 +1577,7 @@ pocl_hsa_submit (_cl_command_node *node, cl_command_queue cq)
   pocl_hsa_device_data_t *d = device->data;
   unsigned added_to_readylist = 0;
 
-  PTHREAD_CHECK(pthread_mutex_lock(&d->list_mutex));
+  PTHREAD_CHECK (pthread_mutex_lock (&d->list_mutex));
 
   node->ready = 1;
   if (pocl_command_is_ready (node->event))
@@ -1735,7 +1760,7 @@ pocl_hsa_launch (pocl_hsa_device_data_t *d, cl_event event)
 
   HSA_CHECK(hsa_memory_allocate (d->kernarg_region,
 				 cached_data->args_segment_size,
-				 &event_data->actual_kernargs));
+				 &event_data->kernargs));
 
   dd->last_queue = (dd->last_queue + 1) % dd->num_queues;
   hsa_queue_t* last_queue = dd->queues[dd->last_queue];
@@ -1761,14 +1786,6 @@ pocl_hsa_launch (pocl_hsa_device_data_t *d, cl_event event)
 	 a single-WI WG. */
       kernel_packet->workgroup_size_x = kernel_packet->workgroup_size_y =
 	kernel_packet->workgroup_size_z = 1;
-
-      if (d->device->device_side_printf)
-	{
-	  pc->printf_buffer = d->printf_buffer;
-	  pc->printf_buffer_capacity = d->device->printf_buffer_size;
-	  bzero (d->printf_write_pos, sizeof (size_t));
-	  pc->printf_buffer_position = d->printf_write_pos;
-	}
     }
   else
     {
@@ -1779,6 +1796,13 @@ pocl_hsa_launch (pocl_hsa_device_data_t *d, cl_event event)
       kernel_packet->workgroup_size_z = run_cmd->pc.local_size[2];
     }
 
+  if (d->device->device_side_printf)
+    {
+      pc->printf_buffer = d->printf_buffer;
+      pc->printf_buffer_capacity = d->device->printf_buffer_size;
+      bzero (d->printf_write_pos, sizeof (size_t));
+      pc->printf_buffer_position = d->printf_write_pos;
+    }
 
   /* TODO: Dynamic WG sizes. */
 
@@ -1808,7 +1832,7 @@ pocl_hsa_launch (pocl_hsa_device_data_t *d, cl_event event)
   HSA_CHECK (
       hsa_signal_create (1, 1, &d->agent, &kernel_packet->completion_signal));
 
-  setup_kernel_args (d, cmd, (char *)event_data->actual_kernargs,
+  setup_kernel_args (d, cmd, event_data,
                      cached_data->args_segment_size, &total_group_size);
 
   kernel_packet->group_segment_size = total_group_size;
@@ -1823,7 +1847,7 @@ pocl_hsa_launch (pocl_hsa_device_data_t *d, cl_event event)
   if (total_group_size > cmd->device->local_mem_size)
     POCL_ABORT ("pocl-hsa: required local memory > device local memory!\n");
 
-  kernel_packet->kernarg_address = event_data->actual_kernargs;
+  kernel_packet->kernarg_address = event_data->kernargs;
 
   typedef union {
     uint32_t header_setup;
@@ -1890,7 +1914,8 @@ pocl_hsa_ndrange_event_finished (pocl_hsa_device_data_t *d, size_t i)
   hsa_signal_destroy (dd->running_signals[i]);
   dd->running_signals[i] = dd->running_signals[dd->running_list_size];
 
-  hsa_memory_free (event_data->actual_kernargs);
+  hsa_memory_free (event_data->kernargs);
+  hsa_memory_free (event_data->context);
 
   POCL_UNLOCK_OBJ (event);
 
