@@ -74,7 +74,7 @@ TCEDevice::TCEDevice(cl_device_id dev, const char* adfName) :
   pthread_mutex_init (&cq_lock, NULL);
   POCL_INIT_LOCK(tce_compile_lock);
   dev->address_bits = 32;
-  dev->autolocals_to_args = 1;
+  dev->autolocals_to_args = POCL_AUTOLOCALS_TO_ARGS_ALWAYS;
   /* This assumes TCE is always Little-endian;
    * needsByteSwap is set up again in TTASimDevice
    * after we know whether ADF is big- or little-endian. */
@@ -480,7 +480,7 @@ void pocl_tce_compile_kernel(_cl_command_node *Command, cl_kernel Kernel,
     AssemblyFileName += "/parallel.tpef";
 
     if (access(AssemblyFileName.c_str(), F_OK) != 0) {
-      Error = snprintf(ByteCode, POCL_FILENAME_LENGTH, "%s%s", CacheDir,
+      Error = snprintf(ByteCode, POCL_FILENAME_LENGTH + 13, "%s%s", CacheDir,
                        POCL_PARALLEL_BC_FILENAME);
       TCEString BuildCmd = Dev->tceccCommandLine(RunCommand, TempDir, ByteCode,
                                                  AssemblyFileName);
@@ -848,8 +848,8 @@ pocl_tce_copy_rect (void *data,
   chunk_info_t *src_chunk = (chunk_info_t*)src_mem_id->mem_ptr;
   chunk_info_t *dst_chunk = (chunk_info_t*)dst_mem_id->mem_ptr;
 
-  size_t src_offset = src_origin[0] + src_row_pitch * (src_origin[1] + src_slice_pitch * src_origin[2]);
-  size_t dst_offset = dst_origin[0] + dst_row_pitch * (dst_origin[1] + dst_slice_pitch * dst_origin[2]);
+  size_t src_offset = src_origin[0] + src_row_pitch * src_origin[1] + src_slice_pitch * src_origin[2];
+  size_t dst_offset = dst_origin[0] + dst_row_pitch * dst_origin[1] + dst_slice_pitch * dst_origin[2];
 
   size_t j, k;
 
