@@ -31,6 +31,7 @@
 #include <string>
 
 #include "TCEString.hh"
+#include "pocl_device.h"
 
 namespace TTAMachine {
   class AddressSpace;
@@ -90,9 +91,10 @@ class TCEDevice {
 
   /* Generates the command line string to execute tcecc to produce the
      kernel binary. */
-  TCEString tceccCommandLine
-    (_cl_command_run *run_cmd, const TCEString& inputSrc, 
-     const TCEString& outputTpef, const TCEString extraParams=TCEString(""));
+  TCEString tceccCommandLine(_cl_command_run *run_cmd, const TCEString &tempDir,
+                             const TCEString &inputSrc,
+                             const TCEString &outputTpef,
+                             const TCEString extraParams = TCEString(""));
 
   bool isMultiCoreMachine() const;
 
