@@ -263,10 +263,20 @@ process_options (const char *options, char *modded_options, char *link_options,
                   *requires_correctly_rounded_sqrt_div = 1;
                 }
             }
+
           if (strstr (cl_parameters, token))
             {
               /* the LLVM API call pushes the parameters directly to the
                  frontend without using -Xclang */
+
+#ifndef LLVM_OLDER_THAN_11_0
+            // LLVM 11 has removed "-cl-denorms-are-zero" option
+            // https://reviews.llvm.org/D69878
+            if (strncmp(token, "-cl-denorms-are-zero", 20) == 0) {
+                token = "-fdenormal-fp-math=positive-zero";
+            }
+#endif
+
             }
           else if (strstr (cl_parameters_not_yet_supported_by_clang, token))
             {
@@ -285,8 +295,13 @@ process_options (const char *options, char *modded_options, char *link_options,
         }
       else if (strncmp (token, "-g", 2) == 0)
         {
+#ifdef LLVM_OLDER_THAN_11_0
           token = "-dwarf-column-info -debug-info-kind=limited " \
 	    "-dwarf-version=4 -debugger-tuning=gdb";
+#else
+          token = "-debug-info-kind=limited " \
+	    "-dwarf-version=4 -debugger-tuning=gdb";
+#endif
         }
       else if (strncmp (token, "-D", 2) == 0 || strncmp (token, "-I", 2) == 0)
         {
