@@ -223,8 +223,8 @@ typedef pthread_mutex_t pocl_lock_t;
 /* Symbol aliases are supported */
 
 #  define POname(name) PO##name
-#  define POdeclsym(name)                      \
-  __typeof__(name) PO##name __attribute__((visibility("hidden")));
+
+#define POdeclsym(name) __typeof__ (name) PO##name;
 #  define POCL_ALIAS_OPENCL_SYMBOL(name)                                \
   __typeof__(name) name __attribute__((alias ("PO" #name), visibility("default")));
 #  define POsymAlways(name) POCL_ALIAS_OPENCL_SYMBOL(name)
@@ -621,6 +621,14 @@ struct pocl_device_ops {
      allocated at the given target. */
   cl_int (*get_builtin_kernel_metadata) (void *data, const char *kernel_name,
                                          pocl_kernel_metadata_t *target);
+
+  /* The device can override this function to perform driver-specific
+   * optimizations to the local size dimensions, whenever the decision
+   * is left to the runtime. */
+  void (*compute_local_size) (cl_device_id dev, size_t global_x,
+                              size_t global_y, size_t global_z,
+                              size_t *local_x, size_t *local_y,
+                              size_t *local_z);
 };
 
 typedef struct pocl_global_mem_t {
@@ -630,6 +638,18 @@ typedef struct pocl_global_mem_t {
   cl_ulong total_alloc_limit;
 } pocl_global_mem_t;
 
+/**
+ * Enumeration for different modes of converting automatic locals
+ */
+typedef enum
+{
+  POCL_AUTOLOCALS_TO_ARGS_NEVER = 0,
+  POCL_AUTOLOCALS_TO_ARGS_ALWAYS = 1,
+  // convert autolocals to args only if there are dynamic local memory function
+  // arguments in the kernel.
+  POCL_AUTOLOCALS_TO_ARGS_ONLY_IF_DYNAMIC_LOCALS_PRESENT = 2,
+} pocl_autolocals_to_args_strategy;
+
 #define NUM_OPENCL_IMAGE_TYPES 6
 
 struct _cl_device_id {
@@ -754,7 +774,7 @@ struct _cl_device_id {
    * the host's system time (pocl_gettimemono_ns). */
   int has_own_timer;
   /* Convert automatic local variables to kernel arguments? */
-  int autolocals_to_args;
+  pocl_autolocals_to_args_strategy autolocals_to_args;
   /* Allocate local buffers device side in the work-group launcher instead of
      having a disjoint physical local memory per work-group or having the
      runtime/driver allocate the local space. */
