@@ -18,8 +18,15 @@
 #include <string.h>
 
 #include "pocl.h"
+#include "pocl_debug.h"
 #include "pocl_file_util.h"
 
+#ifdef __ANDROID__
+
+int pocl_mkstemp(char *path);
+
+#endif
+
 /*****************************************************************************/
 
 int
@@ -100,20 +107,6 @@ pocl_exists(const char* path)
   return !access(path, R_OK);
 }
 
-int
-pocl_filesize(const char* path, uint64_t* res) 
-{
-  FILE *f = fopen(path, "r");
-  if (f == NULL)
-    return -1;
-  
-  fseek(f, 0, SEEK_END);
-  *res = ftell(f);
-  
-  fclose(f);
-  return 0;
-}
-
 int 
 pocl_touch_file(const char* path) 
 {
@@ -127,37 +120,59 @@ pocl_touch_file(const char* path)
 }
 /****************************************************************************/
 
+#define CHUNK_SIZE (2 * 1024 * 1024)
+
 int
 pocl_read_file(const char* path, char** content, uint64_t *filesize) 
 {
   assert(content);
   assert(path);
   assert(filesize);
-  
   *content = NULL;
-  
-  int errcode = pocl_filesize(path, filesize);
-  ssize_t fsize = (ssize_t)(*filesize);
-  if (!errcode) 
+  *filesize = 0;
+
+  /* files in /proc return zero size, while
+     files in /sys return size larger than actual actual content size;
+     this reads the content sequentially. */
+  size_t total_size = 0;
+  size_t actually_read = 0;
+  char *ptr = (char *)malloc (CHUNK_SIZE + 1);
+  if (ptr == NULL)
+    return -1;
+
+  FILE *f = fopen (path, "r");
+  if (f == NULL) {
+    POCL_MSG_ERR ("fopen( %s ) failed\n", path);
+    goto ERROR;
+  }
+
+  do
     {
-      FILE *f = fopen(path, "r");
-      
-      *content = (char*)malloc(fsize+1);
-      
-      size_t rsize = fread(*content, 1, fsize, f);
-      (*content)[rsize] = '\0';
-      if (rsize < (size_t)fsize) 
-        {
-          errcode = -1;
-          fclose(f);
-        } 
-      else 
-        {
-          if (fclose(f))
-            errcode = -1;
-        }
+      char *reallocated = (char *)realloc (ptr, (total_size + CHUNK_SIZE + 1));
+      if (reallocated == NULL)
+        goto ERROR;
+      ptr = reallocated;
+
+      actually_read = fread (ptr + total_size, 1, CHUNK_SIZE, f);
+      total_size += actually_read;
     }
-  return errcode;
+  while (actually_read == CHUNK_SIZE);
+
+  if (ferror (f))
+    goto ERROR;
+
+  if (fclose (f))
+    goto ERROR;
+
+  /* add an extra NULL character for strings */
+  ptr[total_size] = 0;
+  *content = ptr;
+  *filesize = (uint64_t)total_size;
+  return 0;
+
+ERROR:
+  free (ptr);
+  return -1;
 }
 
 
@@ -182,7 +197,10 @@ pocl_write_file(const char *path, const char* content,
         {
           int res = pocl_remove(path);
           if (res)
-            return res;
+            {
+              POCL_MSG_ERR ("pocl_remove(%s) failed\n", path);
+              return res;
+            }
         }
     }
   
@@ -191,22 +209,37 @@ pocl_write_file(const char *path, const char* content,
     f = fopen(path, "a");
   else
     f = fopen(path, "w");
-  
+
   if (f == NULL)
-    return -1;
-  
-  if (fwrite(content, 1, (size_t)count, f) < (size_t)count)
-    return -1;
+    {
+      POCL_MSG_ERR ("fopen(%s) failed\n", path);
+      return -1;
+    }
+
+  if (fwrite (content, 1, (size_t)count, f) < (size_t)count)
+    {
+      POCL_MSG_ERR ("fwrite(%s) failed\n", path);
+      return -1;
+    }
 
   if (fflush (f))
-    return errno;
+    {
+      POCL_MSG_ERR ("fflush() failed\n");
+      return errno;
+    }
 
 #ifdef HAVE_FDATASYNC
   if (fdatasync (fileno (f)))
-    return errno;
+    {
+      POCL_MSG_ERR ("fdatasync() failed\n");
+      return errno;
+    }
 #elif defined(HAVE_FSYNC)
   if (fsync (fileno (f)))
-    return errno;
+    {
+      POCL_MSG_ERR ("fsync() failed\n");
+      return errno;
+    }
 #endif
 
   return fclose(f);
@@ -214,13 +247,17 @@ pocl_write_file(const char *path, const char* content,
 
 /****************************************************************************/
 
+int pocl_rename(const char *oldpath, const char *newpath) {
+  return rename (oldpath, newpath);
+}
+
 int
 pocl_mk_tempname (char *output, const char *prefix, const char *suffix,
                   int *ret_fd)
 {
 #if defined(_MSC_VER) || defined(__MINGW32__)
-  assert (0);
-#elif defined(HAVE_MKOSTEMPS) || defined(HAVE_MKSTEMPS)
+#error "making temporary files on Windows not implemented"
+#elif defined(HAVE_MKOSTEMPS) || defined(HAVE_MKSTEMPS) || defined(__ANDROID__)
   /* using mkstemp() instead of tmpnam() has no real benefit
    * here, as we have to pass the filename to llvm,
    * but tmpnam() generates an annoying warning... */
@@ -230,6 +267,9 @@ pocl_mk_tempname (char *output, const char *prefix, const char *suffix,
   size_t len = strlen (prefix);
   strncpy (output + len, "_XXXXXX", (POCL_FILENAME_LENGTH - len));
 
+#ifdef __ANDROID__
+  fd = pocl_mkstemp (output);
+#else
   if (suffix)
     {
       len += 7;
@@ -245,17 +285,23 @@ pocl_mk_tempname (char *output, const char *prefix, const char *suffix,
     fd = mkostemp (output, O_CLOEXEC);
 #else
     fd = mkstemp (output);
+#endif
 #endif
 
   if (fd < 0)
-    return errno;
+    {
+      POCL_MSG_ERR ("mkstemp() failed\n");
+      return errno;
+    }
 
+  int err = 0;
   if (ret_fd)
     *ret_fd = fd;
   else
-    close (fd);
+    err = close (fd);
+
+  return err ? errno : 0;
 
-  return errno;
 #else
 #error mkostemps() / mkstemps() both unavailable
 #endif
@@ -293,28 +339,44 @@ pocl_write_tempfile (char *output_path, const char *prefix, const char *suffix,
 
   err = pocl_mk_tempname (output_path, prefix, suffix, &fd);
   if (err)
-    return err;
-
+    {
+      POCL_MSG_ERR ("pocl_mk_tempname() failed\n");
+      return err;
+    }
   size_t bytes = (size_t)count;
-  ssize_t res = write (fd, content, bytes);
-  if (res < 0)
-    return errno;
-
-  if ((size_t)res < bytes)
-    return -1;
+  ssize_t res;
+  do
+    {
+      res = write (fd, content, bytes);
+      if (res < 0)
+        {
+          POCL_MSG_ERR ("write(%s) failed\n", output_path);
+          return errno;
+        }
+      else
+        {
+          bytes -= res;
+          content += res;
+        }
+    }
+  while (bytes > 0);
 
 #ifdef HAVE_FDATASYNC
   if (fdatasync (fd))
-    return errno;
+    {
+      POCL_MSG_ERR ("fdatasync() failed\n");
+      return errno;
+    }
 #elif defined(HAVE_FSYNC)
   if (fsync (fd))
     return errno;
 #endif
 
+  err = 0;
   if (ret_fd)
     *ret_fd = fd;
   else
-    close (fd);
+    err = close (fd);
 
-  return errno;
+  return err ? errno : 0;
 }
