@@ -23,11 +23,12 @@
    THE SOFTWARE.
 */
 
+#include "AutomaticLocals.h"
 #include "config.h"
 #include "pocl.h"
 #include "pocl_cache.h"
-#include "pocl_llvm_api.h"
 #include "pocl_file_util.h"
+#include "pocl_llvm_api.h"
 
 #include <string>
 #include <map>
@@ -225,8 +226,7 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
   passes.push_back("workitem-handler-chooser");
   passes.push_back("mem2reg");
   passes.push_back("domtree");
-  if (device->autolocals_to_args)
-    passes.push_back("automatic-locals");
+  passes.push_back("automatic-locals");
 
   if (SPMDDevice) {
     passes.push_back("flatten-inline-all");
@@ -324,6 +324,10 @@ kernel_compiler_passes(cl_device_id device, llvm::Module *input,
       Builder.populateModulePassManager(*Passes);
       continue;
     }
+    if (passes[i] == "automatic-locals") {
+      Passes->add(pocl::createAutomaticLocalsPass(device->autolocals_to_args));
+      continue;
+    }
 
     const PassInfo *PIs = Registry->getPassInfo(StringRef(passes[i]));
     if (PIs) {
@@ -441,6 +445,11 @@ int pocl_llvm_generate_workgroup_function_nowrite(
   llvm::reportAndResetTimings();
 #endif
 
+  // Print loop vectorizer remarks if enabled.
+  if (pocl_get_bool_option("POCL_VECTORIZER_REMARKS", 0) == 1) {
+    std::cout << getDiagString();
+  }
+
   assert(Output != NULL);
   *Output = (void *)ParallelBC;
   ++numberOfIRs;
@@ -625,8 +634,8 @@ int pocl_llvm_codegen(cl_device_id Device, void *Modp, char **Output,
 #ifdef DUMP_LLVM_PASS_TIMINGS
     llvm::reportAndResetTimings();
 #endif
-    std::string O = SOS.str(); // flush
-    const char *Cstr = O.c_str();
+    auto O = SOS.str(); // flush
+    const char *Cstr = O.data();
     size_t S = O.size();
     *Output = (char *)malloc(S);
     *OutputSize = S;
