@@ -36,11 +36,6 @@
 #error aligned malloc unavailable
 #endif
 
-
-#ifdef __GNUC__
-#pragma GCC visibility push(hidden)
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -221,10 +216,6 @@ float half_to_float (uint16_t value);
 }
 #endif
 
-#ifdef __GNUC__
-#pragma GCC visibility pop
-#endif
-
 /* Common macro for cleaning up "*GetInfo" API call implementations.
  * All the *GetInfo functions have been specified to look alike, 
  * and have been implemented to use the same variable names, so this
