@@ -24,6 +24,7 @@
 #include "CompilerWarnings.h"
 IGNORE_COMPILER_WARNING("-Wunused-parameter")
 
+#include "pocl_cl.h"
 #include "pocl_spir.h"
 
 #include "llvm/Pass.h"
@@ -40,6 +41,7 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Module.h"
 
+#include "AutomaticLocals.h"
 #include "LLVMUtils.h"
 #include "Workgroup.h"
 
@@ -54,7 +56,10 @@ namespace {
 
   public:
     static char ID;
-    AutomaticLocals() : ModulePass(ID) {}
+    pocl_autolocals_to_args_strategy autolocals_to_args;
+    AutomaticLocals(pocl_autolocals_to_args_strategy autolocals_to_args =
+                        POCL_AUTOLOCALS_TO_ARGS_ALWAYS)
+        : ModulePass(ID), autolocals_to_args(autolocals_to_args) {}
 
     virtual void getAnalysisUsage(AnalysisUsage &AU) const;
     virtual bool runOnModule(Module &M);
@@ -64,6 +69,11 @@ namespace {
   };
 }
 
+llvm::ModulePass *pocl::createAutomaticLocalsPass(
+    pocl_autolocals_to_args_strategy autolocals_to_args) {
+  return new AutomaticLocals(autolocals_to_args);
+}
+
 char AutomaticLocals::ID = 0;
 static RegisterPass<AutomaticLocals> X("automatic-locals",
 				      "Processes automatic locals");
@@ -75,6 +85,9 @@ AutomaticLocals::getAnalysisUsage(AnalysisUsage &) const {
 bool
 AutomaticLocals::runOnModule(Module &M)
 {
+  if (autolocals_to_args == POCL_AUTOLOCALS_TO_ARGS_NEVER) {
+    return false;
+  }
   bool changed = false;
 
   // store the new and old kernel pairs in order to regenerate
@@ -169,6 +182,21 @@ AutomaticLocals::processAutomaticLocals(Function *F) {
     return F;
   }
 
+  if (autolocals_to_args ==
+      POCL_AUTOLOCALS_TO_ARGS_ONLY_IF_DYNAMIC_LOCALS_PRESENT) {
+    bool NeedsArgOffsets = false;
+    for (auto &Arg : F->args()) {
+      // Check for local memory pointer.
+      llvm::Type *ArgType = Arg.getType();
+      if (ArgType->isPointerTy() && ArgType->getPointerAddressSpace() == 3) {
+        NeedsArgOffsets = true;
+        break;
+      }
+    }
+    if (!NeedsArgOffsets)
+      return F;
+  }
+
   // Create the new function.
   FunctionType *FT =
     FunctionType::get(F->getReturnType(), Parameters, F->isVarArg());
