@@ -42,9 +42,9 @@ struct BreakConstantGEPs : public FunctionPass {
   public:
     static char ID;
     BreakConstantGEPs() : FunctionPass(ID) {}
-    const char *getPassName() const {return "Remove Constant GEP Expressions";}
-    virtual bool runOnFunction (Function & F);
-    virtual void getAnalysisUsage(AnalysisUsage &AU) const {
+    llvm::StringRef getPassName() const override {return "Remove Constant GEP Expressions";}
+    virtual bool runOnFunction (Function & F) override;
+    virtual void getAnalysisUsage(AnalysisUsage &AU) const override {
       // This pass does not modify the control-flow graph of the function
       AU.setPreservesCFG();
     }
