@@ -35,14 +35,17 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 
 #include "llvm/ADT/Statistic.h"
 #include "llvm/Analysis/LoopInfo.h"
-#include "llvm/Support/CommandLine.h"
-#include "llvm/IR/IRBuilder.h"
-#include "llvm/IR/MDBuilder.h"
+#include "llvm/Analysis/PostDominators.h"
+#include "llvm/IR/DIBuilder.h"
 #include "llvm/IR/DataLayout.h"
+#include "llvm/IR/DebugInfoMetadata.h"
+#include "llvm/IR/IRBuilder.h"
 #include "llvm/IR/Instructions.h"
+#include "llvm/IR/IntrinsicInst.h"
+#include "llvm/IR/MDBuilder.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/ValueSymbolTable.h"
-#include "llvm/Analysis/PostDominators.h"
+#include "llvm/Support/CommandLine.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 
 #include "WorkitemLoops.h"
@@ -212,6 +215,16 @@ WorkitemLoops::CreateLoopAround
 
   DTP->runOnFunction(*F);
 
+  /* Collect the basic blocks in the parallel region that dominate the
+     exit. These are used in determining whether load instructions may
+     be executed unconditionally in the parallel loop (see below). */
+  llvm::SmallPtrSet<llvm::BasicBlock *, 8> dominatesExitBB;
+  for (auto bb: region) {
+    if (DT->dominates(bb, exitBB)) {
+      dominatesExitBB.insert(bb);
+    }
+  }
+
   //  F->viewCFG();
   /* Fix the old edges jumping to the region to jump to the basic block
      that starts the created loop. Back edges should still point to the
@@ -308,10 +321,17 @@ WorkitemLoops::CreateLoopAround
   //   !1 = metadata !{metadata !1} <- self-referential root
   loopBranch->setMetadata("llvm.loop", Root);
 
+  auto IsLoadUnconditionallySafe =
+    [&dominatesExitBB](llvm::Instruction *insn) -> bool {
+      assert(insn->mayReadFromMemory());
+      // Checks that the instruction isn't in a conditional block.
+      return dominatesExitBB.count(insn->getParent());
+    };
+
 #ifdef LLVM_OLDER_THAN_8_0
-  region.AddParallelLoopMetadata(Root);
+  region.AddParallelLoopMetadata(Root, IsLoadUnconditionallySafe);
 #else
-  region.AddParallelLoopMetadata(AccessGroupMD);
+  region.AddParallelLoopMetadata(AccessGroupMD, IsLoadUnconditionallySafe);
 #endif
 
   builder.SetInsertPoint(loopEndBB);
@@ -875,6 +895,52 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
 
   BasicBlock &bb = instruction->getParent()->getParent()->getEntryBlock();
   IRBuilder<> builder(&*(bb.getFirstInsertionPt()));
+  Function *FF = instruction->getParent()->getParent();
+  Module *M = instruction->getParent()->getParent()->getParent();
+  LLVMContext &C = M->getContext();
+  const llvm::DataLayout &Layout = M->getDataLayout();
+  DICompileUnit *CU = nullptr;
+  std::unique_ptr<DIBuilder> DB;
+#ifndef LLVM_OLDER_THAN_7_0
+  if (M->debug_compile_units_begin() != M->debug_compile_units_end()) {
+    CU = *M->debug_compile_units_begin();
+    DB = std::unique_ptr<DIBuilder>{new DIBuilder(*M, true, CU)};
+  }
+#endif
+
+  // find the debug metadata corresponding to this variable
+  Value *DebugVal = nullptr;
+  IntrinsicInst *DebugCall = nullptr;
+#ifndef LLVM_OLDER_THAN_7_0
+  if (CU) {
+    for (BasicBlock &BB : (*FF)) {
+      for (Instruction &I : BB) {
+        IntrinsicInst *CI = dyn_cast<IntrinsicInst>(&I);
+        if (CI && (CI->getIntrinsicID() == llvm::Intrinsic::dbg_declare)) {
+          Metadata *Meta =
+              cast<MetadataAsValue>(CI->getOperand(0))->getMetadata();
+          if (isa<ValueAsMetadata>(Meta)) {
+            Value *V = cast<ValueAsMetadata>(Meta)->getValue();
+            if (instruction == V) {
+              DebugVal = V;
+              DebugCall = CI;
+              break;
+            }
+          }
+        }
+      }
+    }
+  }
+#endif
+
+#ifdef DEBUG_WORK_ITEM_LOOPS
+  if (DebugVal && DebugCall) {
+    std::cerr << "### DI INTRIN: \n";
+    DebugCall->dump();
+    std::cerr << "### DI VALUE:  \n";
+    DebugVal->dump();
+  }
+#endif
 
   llvm::Type *elementType;
   if (isa<AllocaInst>(instruction))
@@ -895,9 +961,6 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
       elementType = instruction->getType();
     }
 
-  Module* M = instruction->getParent()->getParent()->getParent();
-  const llvm::DataLayout &Layout = M->getDataLayout();
-
   /* 3D context array. In case the elementType itself is an array or struct,
    * we must take into account it could be alloca-ed with alignment and loads
    * or stores might use vectorized instructions expecting proper alignment.
@@ -988,10 +1051,81 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
      accesses to them. Also, LLVM 3.3 seems to produce illegal
      code at least with Core i5 when aligned only at the element
      size. */
-  Alloca->setAlignment(CONTEXT_ARRAY_ALIGN);
+    Alloca->setAlignment(
+#ifndef LLVM_OLDER_THAN_10_0
+#ifndef LLVM_OLDER_THAN_11_0
+        llvm::Align(
+#else
+        llvm::MaybeAlign(
+#endif
+#endif
+            CONTEXT_ARRAY_ALIGN
+#ifndef LLVM_OLDER_THAN_10_0
+            )
+#endif
+    );
 
-  contextArrays[varName] = Alloca;
-  return Alloca;
+    if (DebugVal && DebugCall && !WGDynamicLocalSize) {
+
+      llvm::SmallVector<llvm::Metadata *, 4> Subscripts;
+      Subscripts.push_back(DB->getOrCreateSubrange(0, WGLocalSizeZ));
+      Subscripts.push_back(DB->getOrCreateSubrange(0, WGLocalSizeY));
+      Subscripts.push_back(DB->getOrCreateSubrange(0, WGLocalSizeX));
+      llvm::DINodeArray SubscriptArray = DB->getOrCreateArray(Subscripts);
+
+      size_t sizeBits;
+#ifndef LLVM_OLDER_THAN_7_0
+      sizeBits = Alloca->getAllocationSizeInBits(M->getDataLayout()).getValueOr(0);
+      assert(sizeBits != 0);
+#endif
+      // if (size == 0) WGLocalSizeX * WGLocalSizeY * WGLocalSizeZ * 8 *
+      // Alloca->getAllocatedType()->getScalarSizeInBits();
+      size_t alignBits = Alloca->getAlignment() * 8;
+
+      Metadata *VariableDebugMeta =
+          cast<MetadataAsValue>(DebugCall->getOperand(1))->getMetadata();
+#ifdef DEBUG_WORK_ITEM_LOOPS
+      std::cerr << "### VariableDebugMeta :  ";
+      VariableDebugMeta->dump();
+      std::cerr << "### sizeBits :  " << sizeBits
+                << "  alignBits: " << alignBits << "\n";
+#endif
+
+      DILocalVariable *LocalVar = dyn_cast<DILocalVariable>(VariableDebugMeta);
+      assert(LocalVar);
+      if (LocalVar) {
+
+#ifdef LLVM_OLDER_THAN_9_0
+        DICompositeType *CT = DB->createArrayType(
+            sizeBits, alignBits, LocalVar->getType().resolve(), SubscriptArray);
+#else
+        DICompositeType *CT = DB->createArrayType(
+            sizeBits, alignBits, LocalVar->getType(), SubscriptArray);
+#endif
+
+#ifdef DEBUG_WORK_ITEM_LOOPS
+        std::cerr << "### DICompositeType:\n";
+        CT->dump();
+#endif
+        DILocalVariable *NewLocalVar = DB->createAutoVariable(
+            LocalVar->getScope(), LocalVar->getName(), LocalVar->getFile(),
+            LocalVar->getLine(), CT, false, LocalVar->getFlags());
+
+        Metadata *NewMeta = ValueAsMetadata::get(Alloca);
+        DebugCall->setOperand(0,
+                              MetadataAsValue::get(M->getContext(), NewMeta));
+
+        MetadataAsValue *NewLV =
+            MetadataAsValue::get(M->getContext(), NewLocalVar);
+        DebugCall->setOperand(1, NewLV);
+
+        DebugCall->removeFromParent();
+        DebugCall->insertAfter(Alloca);
+      }
+    }
+
+    contextArrays[varName] = Alloca;
+    return Alloca;
 }
 
 /**
