@@ -31,12 +31,13 @@
 #include "llvm/ADT/Twine.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Transforms/Utils/ValueMapper.h"
+#include "llvm/IR/IRBuilder.h"
 
 #include "WorkitemHandler.h"
 #include "ParallelRegion.h"
 
 namespace llvm {
-  struct PostDominatorTree;
+  struct PostDominatorTreeWrapperPass;
 }
 
 namespace pocl {
@@ -47,7 +48,8 @@ namespace pocl {
   public:
     static char ID;
 
-  WorkitemLoops() : pocl::WorkitemHandler(ID) {}
+  WorkitemLoops() : pocl::WorkitemHandler(ID),
+                    original_parallel_regions(nullptr) {}
 
     virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const;
     virtual bool runOnFunction(llvm::Function &F);
@@ -60,12 +62,10 @@ namespace pocl {
     typedef std::map<std::string, llvm::Instruction*> StrInstructionMap;
 
     llvm::DominatorTree *DT;
-#ifdef LLVM_OLDER_THAN_3_7
-    llvm::LoopInfo *LI;
-#else
     llvm::LoopInfoWrapperPass *LI;
-#endif
-    llvm::PostDominatorTree *PDT;
+
+    llvm::PostDominatorTreeWrapperPass *PDT;
+
     llvm::DominatorTreeWrapperPass *DTP;
 
     ParallelRegion::ParallelRegionVector *original_parallel_regions;
@@ -76,19 +76,25 @@ namespace pocl {
 
     void FixMultiRegionVariables(ParallelRegion *region);
     void AddContextSaveRestore(llvm::Instruction *instruction);
+    void releaseParallelRegions();
 
-    llvm::Instruction *AddContextSave(llvm::Instruction *instruction, llvm::Instruction *alloca);
-    llvm::Instruction *AddContextRestore
-        (llvm::Value *val, llvm::Instruction *alloca, 
-         llvm::Instruction *before=NULL, 
-         bool isAlloca=false);
-    llvm::Instruction *GetContextArray(llvm::Instruction *val);
+    llvm::Value *GetLinearWiIndex(llvm::IRBuilder<> &builder, llvm::Module *M,
+                                  ParallelRegion *region);
+    llvm::Instruction *AddContextSave(llvm::Instruction *instruction,
+                                      llvm::Instruction *alloca);
+    llvm::Instruction *AddContextRestore(llvm::Value *val,
+                                         llvm::Instruction *alloca,
+                                         bool PoclWrapperStructAdded,
+                                         llvm::Instruction *before = NULL,
+                                         bool isAlloca = false);
+    llvm::Instruction *GetContextArray(llvm::Instruction *val,
+                                       bool &PoclWrapperStructAdded);
 
     std::pair<llvm::BasicBlock *, llvm::BasicBlock *>
     CreateLoopAround
         (ParallelRegion &region, llvm::BasicBlock *entryBB, llvm::BasicBlock *exitBB, 
          bool peeledFirst, llvm::Value *localIdVar, size_t LocalSizeForDim,
-         bool addIncBlock=true);
+         bool addIncBlock=true, llvm::Value *DynamicLocalSize=NULL);
 
     llvm::BasicBlock *
       AppendIncBlock
@@ -99,6 +105,11 @@ namespace pocl {
 
     bool ShouldNotBeContextSaved(llvm::Instruction *instr);
 
+    llvm::Type *RecursivelyAlignArrayType(llvm::Type *ArrayType,
+                                          llvm::Type *ElementType,
+                                          size_t Alignment,
+                                          const llvm::DataLayout &Layout);
+
     std::map<llvm::Instruction*, unsigned> tempInstructionIds;
     size_t tempInstructionIndex;
     // An alloca in the kernel which stores the first iteration to execute
