@@ -1,9 +1,8 @@
 #ifndef POCL_LINKER_H
 #define POCL_LINKER_H
 
-#include "config.h"
-
 #include "llvm/IR/Module.h"
+#include "llvm/ADT/StringRef.h"
 
 #ifdef __GNUC__
 #pragma GCC visibility push(hidden)
@@ -15,8 +14,13 @@
  * in krn from lib, cloning as needed. For big modules,
  * this is faster than calling llvm::Linker and then
  * running DCE.
+ *
+ * log is used to report errors if we run into undefined symbols
  */
-void link(llvm::Module *krn, const llvm::Module *lib);
+int link(llvm::Module *krn, const llvm::Module *lib, std::string &log);
+
+int copyKernelFromBitcode(const char *name, llvm::Module *parallel_bc,
+                          const llvm::Module *program);
 
 #ifdef __GNUC__
 #pragma GCC visibility pop
