@@ -158,7 +158,7 @@ main(void)
         else
           return EXIT_FAILURE;
     }
-    catch (cl::Error err) {
+    catch (cl::Error &err) {
          std::cerr
              << "ERROR: "
              << err.what()
